c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine remesh(Nb,Nq)
      implicit none

      include 'max.inc'
      include 'data.inc'
      include 'param.inc'
      include 'files_out.inc'
c     Purpose: to fit the given spectral data over a different pressure grid
c     than the one that is provided into the main data file.

      integer i,j,band,quad,m2,Nb,Nq,f
      double precision pres2(0:Nmax)
      double precision pres_gcm(0:Nmax)
      double precision e2(0:Nmax)
      double precision alt2(0:Nmax)
      double precision temp2(0:Nmax),grav2(0:Nmax)
      double precision dens2(0:Nmax),adia2(0:Nmax)
      double precision zeta2(0:Nmax),capa2(0:Nmax)

      double precision ka_gaz2(1:Nbmx,1:Nmax,Nqmx)
      double precision kd_gaz2(1:Nbmx,1:Nmax,Nqmx)
      double precision g_gaz2(1:Nbmx,1:Nmax,Nqmx)

      double precision ka_aero2(1:Nbmx,1:Nmax,Nqmx)
      double precision kd_aero2(1:Nbmx,1:Nmax,Nqmx)
      double precision g_aero2(1:Nbmx,1:Nmax,Nqmx)

      double precision ka_cloud2(1:Nbmx,1:Nmax,Nqmx)
      double precision kd_cloud2(1:Nbmx,1:Nmax,Nqmx)
      double precision g_cloud2(1:Nbmx,1:Nmax,Nqmx)

      double precision k_in(1:Nmax),k_out(1:Nmax)
      integer i_start(1:Nmax)
      integer i_end(1:Nmax)
      integer i2
      double precision alpha,om,dalt
c     Order of magnitude of the maximum difference between ground
c     pressures for the initial pressure grid and the pressure grid
c     over which interpolation is required:
      parameter(om=10.0D+0)

c     Reading the final mesh
      call read_pres2(m2,pres2,pres_gcm)
      alpha=dabs(pres2(0)/pres(0))
      if ((alpha.gt.om).or.(alpha.lt.1.0D+0/om)) then
         call screen(1129,0,0,0.0D+0,0.0D+0)
         stop
      endif
c     Debug
c      do i=0,m2-1
c         write(*,*) 'pres2(',i,')=',pres2(i)
c      enddo
c     Debug

c     Computing the final vertical mesh (altitude)
      call calc_e2(m2,e2,pres2,pres_gcm,alt2)
c     Linear interpolation in altitude for temperatures, densities, gravity, etc
      call lin_interp_rm(m2,alt2,temp,temp2)
      call lin_interp_rm(m2,alt2,dens,dens2)
      call lin_interp_rm(m2,alt2,adia,adia2)
      call lin_interp_rm(m2,alt2,grav,grav2)
      call lin_interp_rm(m2,alt2,zeta,zeta2)
      call lin_interp_rm(m2,alt2,capa,capa2)

c     -----------------------------------
c     Solutions to known issues:
c       + issue 1: density may decrease below 0 for very high altitudes
      do i=0,m2-1
         if (dens2(i).lt.0.0D+0) then
c     Looking for last positive value
            f=0
            do j=m2-1,0,-1
               if (dens2(j).gt.0.0D+0) then
                  dens2(i)=dens2(j)
                  f=1
                  goto 321
               endif
            enddo
            if (f.eq.0) then
               call screen(1141,0,0,0.0D+0,0.0D+0)
               stop
            endif
         endif
 321     continue
      enddo

c       + issue 2: ground temperature lower than first gas temperature
      if (Tsol.lt.temp2(0)) then
         Tsol=temp2(0)
      endif
c     -----------------------------------

c     Identification of the (old) layers over which the (new)
c     layers are dispatched.
      call find_tab(m2,e2,i_start,i_end)

c     Debug
c      open(12,file='./results/mesh2.txt')
c      write(12,*) m2
c      do i2=0,m2-1
c         write(12,*) i2,pres2(i2),alt2(i2),i_start(i2),
c     &        i_end(i2),temp2(i2)
c      enddo
c      close(12)
c     Debug

c     Distribution of absorption and scattering optical depths
c     from the old grid to the new grid.
      do band=1,Nb
         do quad=1,Nq
c     Gas absorption coefficient
            call set_one_NbNq(band,quad,m,ka_gaz,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,ka_gaz2)
c     Gas scattering coefficient
            call set_one_NbNq(band,quad,m,kd_gaz,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,kd_gaz2)
c     Gas asymetry parameter
            call set_one_NbNq(band,quad,m,g_gaz,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,g_gaz2)
c     Aerosols absorption coefficient
            call set_one_NbNq(band,quad,m,ka_aero,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,ka_aero2)
c     Aerosols scattering coefficient
            call set_one_NbNq(band,quad,m,kd_aero,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,kd_aero2)
c     Aerosols asymetry parameter
            call set_one_NbNq(band,quad,m,g_aero,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,g_aero2)
c     Clouds absorption coefficient
            call set_one_NbNq(band,quad,m,ka_cloud,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,ka_cloud2)
c     Clouds scattering coefficient
            call set_one_NbNq(band,quad,m,kd_cloud,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,kd_cloud2)
c     Clouds asymetry parameter
            call set_one_NbNq(band,quad,m,g_cloud,k_in)
            call  distri(m,m2,e,e2,i_start,i_end,k_in,k_out)
            call set_two_NbNq(band,quad,m2,k_out,g_cloud2)
         enddo                  ! sur quad
      enddo                     ! sur band


c     In the case alt2(0) is not zero, every altitude has to be modified
c     so that alt2(0)=0. Only because the Monte-Carlo core does not
c     like to work with negative altitudes...
      dalt=-alt2(0)
c     Debug
c      write(*,*) 'dalt=',dalt
c     Debug
      if (dalt.ne.0.0D+0) then
         do i=0,m2-1
            alt2(i)=alt2(i)+dalt
         enddo ! i
         do i=0,m2
            e2(i)=e2(i)+dalt
         enddo ! i
      endif
c     Debug
c      do i=0,m2
c         write(*,*) 'e2(',i,')=',e2(i)
c      enddo
c      stop
c     Debug

c     The consequence of this is that altitude zero is no longer the reference
c     altitude of the planet. It is only the altitude of the ground.
      
c     Some extra tests, just to check everything is fine with the new data
      do i=1,m2-1
         if (pres2(i).gt.pres2(i-1)) then
            call screen(1132,i,0,pres2(i),pres2(i-1))
            stop
         endif
         if (alt2(i).lt.alt2(i-1)) then
            call screen(1133,i,0,alt2(i),alt2(i-1))
            stop
         endif
      enddo
      do i=0,m2-1
         if (pres2(i).lt.0.0D+0) then
            call screen(1134,i,0,pres2(i),0.0D+0)
            stop
         endif
         if (alt2(i).lt.0.0D+0) then
            call screen(1135,i,0,alt2(i),0.0D+0)
            stop
         endif
         if (temp2(i).lt.0.0D+0) then
            call screen(1136,i,0,temp2(i),0.0D+0)
            stop
         endif
         if (dens2(i).lt.0.0D+0) then
            call screen(1137,i,0,dens2(i),0.0D+0)
            stop
         endif
         if (grav2(i).lt.0.0D+0) then
            call screen(1138,i,0,grav2(i),0.0D+0)
            stop
         endif
         if (zeta2(i).lt.0.0D+0) then
            call screen(1139,i,0,zeta2(i),0.0D+0)
            stop
         endif
         if (capa2(i).lt.0.0D+0) then
            call screen(1140,i,0,capa2(i),0.0D+0)
            stop
         endif
      enddo


c     Replacement of old data by interpolated data
      m=m2
      do i=0,m2-1
         alt(i)=alt2(i)
         pres(i)=pres2(i)
         temp(i)=temp2(i)
         dens(i)=dens2(i)
         adia(i)=adia2(i)
         grav(i)=grav2(i)
         zeta(i)=zeta2(i)
         capa(i)=capa2(i)
      enddo
      do i=0,m2
         e(i)=e2(i)
      enddo

c     Idem for spectroscopic data
      do band=1,Nb
         do quad=1,Nq
            do i=1,m
               ka_gaz(band,i,quad)=ka_gaz2(band,i,quad)
               kd_gaz(band,i,quad)=kd_gaz2(band,i,quad)
               g_gaz(band,i,quad)=g_gaz2(band,i,quad)
               ka_aero(band,i,quad)=ka_aero2(band,i,quad)
               kd_aero(band,i,quad)=kd_aero2(band,i,quad)
               g_aero(band,i,quad)=g_aero2(band,i,quad)
               ka_cloud(band,i,quad)=ka_cloud2(band,i,quad)
               kd_cloud(band,i,quad)=kd_cloud2(band,i,quad)
               g_cloud(band,i,quad)=g_cloud2(band,i,quad)
            enddo ! i
         enddo ! quad
      enddo ! band

c     Saving interpolated spectroscopic data in
c     separate data file (needed for CRE equilibrium for instance)
      call write_atmos_data(Nb,Nq)

c     Post-meshing step
      call postmesh()

c     Debug
c      stop
c     Debug

      return
      end
