c     Copyright (C)
c     2000 -2013 Universite Paul Sabatier,
c     2000 -2013 Centre National de la Recherche Scientifique
c     2000 -2013 Ecole des Mines d'Albi
c     2014-2018 Msostar
c
c     This program is free software; you can redistribute it and/or modify
c     it under the terms of the GNU General Public License as published by
c     the Free Software Foundation; either version 2, or (at your option)
c     any later version.

c     This program is distributed in the hope that it will be useful,
c     but WITHOUT ANY WARRANTY; without even the implied warranty of
c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
c     GNU General Public License for more details.

c     You should have received a copy of the GNU General Public License
c     along with this program; if not, write to the Free Software
c     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
c     USA.
c
      subroutine screen(code,arg_int1,arg_int2,arg_real1,arg_real2)

      implicit none

c     Cette routine gre les messages affichs  l'cran en cours d'excution
c
c     Entres :
c       + code : entier permettant de prciser quel message doit tre affich
c       + arg_int1  : argument ENTIER 1 (par exemple md le numro de la maille de dpart du rayon)
c       + arg_int2  : argument ENTIER 2 (par exemple paq le numro du rayon en cours de tirage)
c       + arg_real1 : argument REEL 1
c       + arg_real2 : argument REEL 2
c
c     Plages de 'code' :
c     + plage 1-100 : affichages du code principal
c     + plage 1000-1100 : erreurs de dclaration d'options
c


      include 'max.inc'
      include 'param.inc'
      include 'data.inc'
      include 'options.inc'
      include 'files_in.inc'
      include 'files_out.inc'
      integer code,arg_int1,arg_int2,strlen
      double precision arg_real1,arg_real2
      integer seed,iseed
      integer Nb,Nq,band,quad
      integer*8 day0,month0,year0
      integer hour0,min0,sec0
      common/comsee/iseed

c     Affichage de l'tat d'avancement du calcul
      if (apc.eq.1) then

         if (code.eq.1) then ! Plage 1-100
            if (arg_int1.eq.0) then
               write(*,*) 'Computation in progress for the ground'
            else if (arg_int1.eq.m+1) then
c     Dans le cas o KARINE utilise l'applet java pour afficher la progression du
c     calcul, il faut crire une ligne supplmentaire  la fin pour revenir  la
c     ligne dans la console.
               if (javac.eq.1) then
                  write(*,*) 
               endif
               write(*,*) 'Computation in progress for space'
            else
               if (javac.eq.1) then
                  call overwrite_progress_line(arg_int1)
               else
                  write(*,*) 'Computation in progress for layer',
     &                 arg_int1,' / ',m
               endif
            endif
         else if (code.eq.10) then
            write(*,*) 'Reading spectroscopic data file found at:'
            write(*,*) file_ray(1:strlen(file_ray))
         else if (code.eq.11) then
            write(*,*) 'Post-processing'
         else if (code.eq.12) then
            write(*,*) 'Spectral optimization using NER estimation'
         else if (code.eq.13) then
            write(*,*) 'Looking for known errors'
         else if (code.eq.14) then
            write(*,*) 'No error found, please fasten your seat belt'
         else if (code.eq.15) then
            write(*,*) 'Error found, the program will stop now'
         else if (code.eq.16) then
            write(*,*) 'Resuming previous run'
         else if (code.eq.17) then
            write(*,*) 'Previous run was interrupted, but no resuming'
         else if (code.eq.18) then
            write(*,*) 'Previous run was finished'
         else if (code.eq.19) then
            write(*,*) 'A new computation will start now'
         else if (code.eq.20) then
            call convert_date(date0,day0,month0,year0,
     &           hour0,min0,sec0)
            write(*,*) 'Resuming computation started on ',day0,'/',
     &           month0,'/',year0,' @ ',hour0,':',
     &           min0,':',sec0
         else if (code.eq.100) then
            write(*,*) 'End of file'
            open(10,file='./temporary/coupure.dat')
            write(10,*) 1
            close(10)
         else if (code.eq.2) then
            write(*,*) 'Initial seed:',arg_int1
         else if (code.eq.3) then
            write(*,*) 'Last seed:',iseed
         else if (code.eq.21) then
            write(*,*) 'Minimum integration wavelength',
     &           arg_real1,' microns'
         else if (code.eq.22) then
            write(*,*) 'Maximum integration wavelength',
     &           arg_real1,' microns'
         else if (code.eq.23) then
            write(*,*) 'Number of narrowbands to use :',arg_int1
         else if (code.eq.24) then
            if (javac.eq.1) then
               call overwrite_init_line(arg_int1)
            else
               write(*,*) 'Computation in progress for narrowband '
     &              ,arg_int1,' (',nb_min,' -',nb_max,' )'
c     &           ,' [',no_lo(band),'-',no_hi(band),'] cm-1'
c     &           ,' [',lambda_lo(band),'-',lambda_hi(band),'] m'
            endif
            if ((arg_int1.eq.nb_max).and.(javac.eq.1)) then
               write(*,*)
            endif
         else if (code.eq.25) then
            write(*,*) 'Uniform spectral pdf required'
         else if (code.eq.26) then
            write(*,*) 'Using pdf recorded in file'
            write(*,*) file_freq_opti(1:strlen(file_freq_opti))
         else if (code.eq.27) then
            write(*,*) 'The required pdf does not match input data'
            write(*,*) 'Failed using pdf stored in file'
            write(*,*) file_freq_opti
         else if (code.eq.28) then
            write(*,*) 'Storing pdf in file:'
            write(*,*) file_freq_opti(1:strlen(file_freq_opti))
         else if (code.eq.29) then
            write(*,*) 'First use of KARINE version 3.4.5'
            write(*,*) 'Congratulations !'
         else if (code.eq.30) then
            write(*,*) 'Computation of pdf and cumulatives'
         else if (code.eq.31) then
            write(*,*) 'Computation of acceleration arrays'
         else if (code.eq.33) then
            write(*,*) 'Spectral optimization using fluxes estimation'
         else if (code.eq.34) then
            write(*,*) 'Interpolation over new pressure grid found at:'
            write(*,*) file_meshGCM(1:strlen(file_meshGCM))
         else if (code.eq.35) then
            write(*,*) 'Saving interpolated data in file:'
            write(*,*) file_interp(1:strlen(file_interp))
         else if (code.eq.36) then
            write(*,*) 'Using phase function file found at:'
            write(*,*) file_phase(1:strlen(file_phase))
         else if (code.eq.37) then
            write(*,*) 'Validity check for all results'
         else if (code.eq.38) then
            write(*,*) 'Immediate stop required'
         else if (code.eq.39) then
            write(*,*) 'Karine is resuming now'
         else if (code.eq.40) then
            write(*,*) 'Now computing A,C,D,E and F matrices'
         else if (code.eq.41) then
            write(*,*) 'Saving d(ksi)/d(T) file'
         else if (code.eq.42) then
            write(*,*) 'Saving sums of d(NER)/d(T) file'
         else if (code.eq.43) then
            write(*,*) 'Saving A, C, D, E and F matrices'
         else if (code.eq.44) then
            write(*,*) 'Please refer to the documentation'
         endif
         
      endif
      
c     Affichage des messages d'erreur
      if (code.eq.1001) then
         write(*,*) 'Wrong option "display error messages"'
      endif
      
      if (amsg.eq.1) then
         
         if (code.eq.1002) then ! Plage 1000-1100 : dclaration des options
            write(*,*) 'Wrong option "fixed angle for boundaries em." '
         else if (code.eq.1003) then
            write(*,*) 'Wrong option "lambertian ang. pdf for bound."'
         else if (code.eq.1004) then
            write(*,*) 'Wrong option "isotropic ang. pdf for bound."'
         else if (code.eq.1005) then
            write(*,*) 'Wrong option "reflexion at boundaries"'
         else if (code.eq.1006) then
            write(*,*) 'Wrong option "fixed angle for gas emission"'
         else if (code.eq.1007) then
            write(*,*) 'Wrong option "lambertian ang. pdf for gas"'
         else if (code.eq.1008) then
            write(*,*) 'Wrong option "isotropic ang. pdf for gas"'
         else if (code.eq.1009) then
            write(*,*) 'Wrong option "emission algorithm for gas"'
         else if (code.eq.1010) then
            write(*,*) 'Wrong option "phase function"'
         else if (code.eq.1011) then
            write(*,*) 'Wrong option "NER spectral optimization"'
         else if (code.eq.1012) then
            write(*,*) 'Wrong option "display progress"'
         else if (code.eq.1013) then
            write(*,*) 'Wrong option "troncation after Ndiff scatt."'
         else if (code.eq.1014) then
            write(*,*) 'Wrong option "spectral optimization"'
         else if (code.eq.1015) then
            write(*,*) 'Wrong option "saving monochromatic results"'
         else if (code.eq.1016) then
            write(*,*) 'Wrong option "optimized angular pdf"'
         else if (code.eq.1017) then
            write(*,*) 'Inconsistency between cmc=0 and mco=3'
         else if (code.eq.1018) then
            write(*,*) 'Wrong option "MC computation"'
         else if (code.eq.1019) then
            write(*,*) 'Wrong option "integ. overd the whole spectrum"'
         else if (code.eq.1020) then
            write(*,*) 'Wrong option "saving intermediate results"'
         else if (code.eq.1021) then
            write(*,*) 'Wrong option "computation of mono. NER"'
         else if (code.eq.1022) then
            write(*,*) 'Wrong option "Monte-Carlo optimization"'
         else if (code.eq.1023) then
            write(*,*) 'Wrong option "fluxes spectral optimization"'
         else if (code.eq.1024) then
            write(*,*) 'Wrong option "blackbody intensity computation"'
         else if (code.eq.1025) then
            write(*,*) 'Wrong option for the random number generator'
         else if (code.eq.1026) then
            write(*,*) 'Wrong option for interpolation over a new grid'
         else if (code.eq.1027) then
            write(*,*) 'Wrong option for conservation of absorbant qty'
         else if (code.eq.1028) then
            write(*,*) 'Wrong option for computation of d(NER)/d(T)'
         else if (code.eq.1029) then
            write(*,*) 'Wrong option for saving d(ksi)/d(T)'
         else if (code.eq.1030) then
            write(*,*) 'Wrong option for saving sums of d(NER)/d(T)'
         else if (code.eq.1031) then
            write(*,*) 'Wrong option for saving A, C, D, E, F matrices'
         else if (code.eq.1032) then
            write(*,*) 'Wrong option for computation of d(NER)/d(x)'
         else if (code.eq.1033) then
            write(*,*) 'Wrong option for saving d(NER)/d(x)'
         else if (code.eq.1034) then
            write(*,*) 'Wrong option for saving d(ksi)/d(x)'
         else if (code.eq.1035) then
            write(*,*) 'Wrong option for Planck/user-defined fluxes'
         endif

         
         if (code.eq.1101) then ! Plage 1100-1200 : erreurs en cours d'excution
            write(*,*) 'The index of the boundary is wrong in "wall"'
         else if (code.eq.1102) then
            write(*,*) 'ambiguous angular pdf'
         else if (code.eq.1103) then
            write(*,*) 'The second input argument in routine'
            write(*,*) '"pdf_angulaire" is not allowed'
         else if (code.eq.1104) then
            write(*,*) 'WARNING'
            write(*,*) 'Dimension of array "ray" is too low'
            write(*,*) 'Computation will be stopped'
            write(*,*) 'Try again, using a bigger walue for',
     &           ' DiffMax.'
            write(*,*)
         else if (code.eq.1105) then
            write(*,*) 'input argument j "wall" is wrong'
         else if (code.eq.1106) then
            write(*,*) 'WARNING'
            write(*,*) 'Index Pmax is too low'
            write(*,*) 'Try again, using a bigger value for Pmax'
            write(*,*)
         else if (code.eq.1107) then
            write(*,*) 'z_abs < 0 in routine abs_gaz'
         else if (code.eq.1108) then
            write(*,*) 'z1 < 0 in routine initz'
         else if (code.eq.1111) then
            write(*,*) 'Inconsistency in spectral intervals'
            write(*,*) 'number ',arg_int1,' and ',arg_int2
            write(*,*) 'lambda_hi(',arg_int1,')=',lambda_hi(arg_int1)
            write(*,*) 'lambda_lo(',arg_int2,')=',lambda_lo(arg_int2)
         else if (code.eq.1112) then
            write(*,*) 'k was not found between points',
     &           arg_int1,' and',arg_int2
         else if (code.eq.1113) then
            write(*,*) 'optb(',arg_int1,',',arg_int2,')=',arg_real1
         else if (code.eq.1114) then
            write(*,*) 'opth(',arg_int1,',',arg_int2,')=',arg_real1
         else if (code.eq.1115) then
            write(*,*) 'For R=',arg_real1
            write(*,*) 'k should be between'
            write(*,*) 'points',arg_int1,' and',arg_int2
         else if (code.eq.1116) then
            write(*,*) 'NaN in computed pdf'
            write(*,*) 'The program will be stopped'
         else if (code.eq.1117) then
            write(*,*) 'Error in find_z_i1: i=',arg_int1
            write(*,*) 'z=',arg_real1,' alt(m-1)=',arg_real2
         else if (code.eq.1118) then
            write(*,*) 'Too many consecutive null random numbers'
            write(*,*) 'The program will stop'
         else if (code.eq.1119) then
            write(*,*) 'Error in find_pres_i1: i=',arg_int1
            write(*,*) 'n=',arg_int2
            write(*,*) 'p=',arg_real1,' pres(',arg_int1,')=',arg_real2
         else if (code.eq.1120) then
            write(*,*) 'Error in find_e_i1: i=',arg_int1
         else if (code.eq.1121) then
            write(*,*) 'Distribution error'
            write(*,*) 'Total over old grid=',arg_real1
            write(*,*) 'Total over new grid=',arg_real2
            write(*,*) 'Relative difference=',arg_real2/arg_real1
         else if (code.eq.1122) then
            write(*,*) 'k(1)=',arg_real1,
     &           ' k2(',arg_int1,')=',arg_real2
         else if (code.eq.1123) then
            write(*,*) 'H=',arg_real1,' H2=',arg_real2
         else if (code.eq.1124) then
            write(*,*) 'k2(',arg_int1,')=',arg_real1
         else if (code.eq.1125) then
            write(*,*) arg_int1,' k2(',arg_int2,')=',arg_real1
         else if (code.eq.1126) then
            write(*,*) 'i_start(',arg_int1,')=',arg_real1,
     &           ' i_end(',arg_int1,')=',arg_real2
         else if (code.eq.1127) then
            write(*,*) 'e(',arg_int1,')-e2(',arg_int2,')=',arg_real1
         else if (code.eq.1128) then
            write(*,*) 'e2(',arg_int1,')-e(',arg_int2,')=',
     &           arg_real1
         else if (code.eq.1129) then
            write(*,*) 'Error during interpolation over new grid'
            write(*,*) 'Pressure grids seem inconsistent'
            write(*,*) 'Please check files:'
            write(*,*) '+ atmos_data.txt'
            write(*,*) '+ meshGCM.txt'
         else if (code.eq.1130) then
            write(*,*) 'Error in find_tab_i1: i=',arg_int1
         else if (code.eq.1131) then
            write(*,*) 'Error in the computation of ksi:'
            write(*,*) 'NER(',arg_int1,',',arg_int2,',',int(arg_real1)
     &           ,')=',arg_real2
         else if (code.eq.1132) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'pres2(',arg_int1,')=',arg_real1
            write(*,*) 'pres2(',arg_int1-1,')=',arg_real2
         else if (code.eq.1133) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'alt2(',arg_int1,')=',arg_real1
            write(*,*) 'alt2(',arg_int1-1,')=',arg_real2
         else if (code.eq.1134) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'pres2(',arg_int1,')=',arg_real1
         else if (code.eq.1135) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'alt2(',arg_int1,')=',arg_real1
         else if (code.eq.1136) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'temp2(',arg_int1,')=',arg_real1
         else if (code.eq.1137) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'dens2(',arg_int1,')=',arg_real1
         else if (code.eq.1138) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'grav2(',arg_int1,')=',arg_real1
         else if (code.eq.1139) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'zeta2(',arg_int1,')=',arg_real1
         else if (code.eq.1140) then
            write(*,*) 'Error during interpolation of altitudes:'
            write(*,*) 'capa2(',arg_int1,')=',arg_real1
         else if (code.eq.1141) then
            write(*,*) 'Could not find any positive value for density'
         else if (code.eq.1142) then
            write(*,*) 'NaN detected within interpolated spectral data'
            write(*,*) 'Interpolated data file could not be written'
         else if (code.eq.1143) then
            write(*,*) 'GCM pressure grid file is inconsistent:'
            write(*,*) 'pres_inter(',arg_int1,')=',arg_real1
            write(*,*) 'pres_inter(',arg_int1-1,')=',arg_real2
         else if (code.eq.1144) then
            write(*,*) 'GCM pressure grid file not found:'
            write(*,*) file_meshGCM(1:strlen(file_meshGCM))
         else if (code.eq.1145) then
            write(*,*) 'Phase function file not found:'
            write(*,*) file_phase(1:strlen(file_phase))
         else if (code.eq.1146) then
            write(*,*) 'Could not find input data file:'
            write(*,*) file_ray(1:strlen(file_ray))
         else if (code.eq.1147) then
            write(*,*) 'Resuming previous run is required'
            write(*,*) 'However, at least one record file is missing'
         else if (code.eq.1148) then
            write(*,*) 'Could not find "data.in" file'
         else if (code.eq.1149) then
            write(*,*) 'Could not find "options.in" file'
         else if (code.eq.1150) then
            write(*,*) 'Could not find spectral pdf file:'
            write(*,*) file_freq_opti(1:strlen(file_freq_opti))
         else if (code.eq.1151) then
            write(*,*) 'Could not find random seed file:'
            write(*,*) 'seed0'
         else if (code.eq.1152) then
            write(*,*) 'd(ka)/d(T) file not found:'
            write(*,*) file_dkadT(1:strlen(file_dkadT))
         else if (code.eq.1153) then
            write(*,*) 'd(ka)/d(x) file not found:'
            write(*,*) file_dkadx(1:strlen(file_dkadx))
         else if (code.eq.1154) then
            write(*,*) 'Error: alt(',arg_int1,')=',alt(arg_int1),
     &           ' <= alt(',arg_int2,')=',alt(arg_int2)
         else if (code.eq.1155) then
            write(*,*) 'Error: e(',arg_int1,')=',e(arg_int1),
     &           ' <= e(',arg_int2,')=',e(arg_int2)
         endif

         if (code.eq.1201) then ! Plage 1200-1300 : dpouillement des rsultats
            write(*,*) 'NER(',arg_int1,',',arg_int2,') and NER(',
     &           arg_int2,',',arg_int1,') are not symetrical'
         else if (code.eq.1202) then
            write(*,*) 'No invalid result was found'
         else if (code.eq.1203) then
            write(*,*) 'Invalid results were found'
         endif

         if (code.eq.1301) then ! Plage 1300-1400 : dclaration des donnes
            write(*,*) 'Error : ground emissivity'
         else if (code.eq.1302) then
            write(*,*) 'Error : space emissivity'
         else if (code.eq.1303) then
            write(*,*) 'Error : emission angle for boundaries'
         else if (code.eq.1304) then
            write(*,*) 'Error : emission angle for gas volumes'
         else if (code.eq.1305) then
            write(*,*) 'Error : lambda_min > lambda_max'
         else if (code.eq.1307) then
            write(*,*) 'Error : ksi_min for ner'
         else if (code.eq.1308) then
            write(*,*) 'Error : ksi_min for fluxes'
         else if (code.eq.1309) then
            write(*,*) 'Error : probability for up/down propagation'
         else if (code.eq.1310) then
            write(*,*) 'Error : number of bundles per layer'
         else if (code.eq.1311) then
            write(*,*) 'Error : number of scatt. events for truncation'
         else if (code.eq.1313) then
            write(*,*) 'Error : ksi_min for net fluxes'
         else if (code.eq.1314) then
            write(*,*) 'Pdfh layer',arg_int1,' has been set to uniform'
         else if (code.eq.1315) then
            write(*,*) 'Pdfb layer',arg_int1,' has been set to uniform'
         else if (code.eq.1316) then
            write(*,*) 'Error: number of angular sectors is negative'
         else if (code.eq.1317) then
            write(*,*) 'Error: Nsec > Nsec_mx, please adjust Nsec_mx'
         else if (code.eq.1318) then
            write(*,*) 'Error: Nsec is odd, please adjust Nsec'
         endif

         if (code.eq.1401) then ! Plage 1400-1500 : variables importantes
            write(*,*) 'Error : m=',arg_int1
         else if (code.eq.1402) then
            write(*,*) 'Error : H=',arg_real1
         else if (code.eq.1403) then
            write(*,*) 'm=',arg_int1,' while Nmax=',arg_int2
         else if (code.eq.1404) then
            write(*,*) 'Nb=',arg_int1,' while Nbmx=',arg_int2
         else if (code.eq.1405) then
            write(*,*) 'Nq=',arg_int1,' while Nqmx=',arg_int2
         endif

         if (code.eq.1501) then ! Plage 1500-1600 : warnings sur taille mmoire des rsultats
            write(*,*) 'Warning:'
            if (arg_real1.lt.1024.0D+0) then
               write(*,*) 'Total size for d(ksi)/d(T) files will be:',
     &              arg_real1,' Mb'
            else
               write(*,*) 'Total size for d(ksi)/d(T) files will be:',
     &              arg_real1/1024.0D+0,' Gb'
            endif
         else if (code.eq.1502) then
            write(*,*) 'Warning:'
            if (arg_real1.lt.1024.0D+0) then
               write(*,*) 'Total size for d(ksi)/d(X) files will be:',
     &              arg_real1,' Mb'
            else
               write(*,*) 'Total size for d(ksi)/d(X) files will be:',
     &              arg_real1/1024.0D+0,' Gb'
            endif
         else if (code.eq.1503) then
            write(*,*) 'Warning:'
            if (arg_real1.lt.1024.0D+0) then
               write(*,*) 'Total size for d(NER)/d(X) files will be:',
     &              arg_real1,' Mb'
            else
               write(*,*) 'Total size for d(NER)/d(X) files will be:',
     &              arg_real1/1024.0D+0,' Gb'
            endif
         endif

      endif

 1234 continue
      return
      end
